package main

import (
	"errors"
	"flag"
	"fmt"
	"sync"
	"net"
	"time"
	"bufio"
	"strings"
)

type RedisConfig struct {
	host   string
	port   string
	passwd string
}

type InterfaceConn struct {
	conn    net.Conn
	proxyId string
}

func getRedisConfig() (error, *RedisConfig) {
	host := flag.String("host", "127.0.0.1", "redis-server host")
	port := flag.String("port", "6379", "redis-server port")
	pwd := flag.String("passwd", "", "redis-server password")
	flag.Parse()

	if *host == "" || *pwd == "" {
		flag.Usage()
		return errors.New("host or passwd is null"), nil
	}
	return nil, &RedisConfig{host: *host, port: *port, passwd: *pwd}
}


func getNewConn(config *RedisConfig) *InterfaceConn {
	c, err := net.DialTimeout("tcp", config.host + ":" + config.port, 5 * time.Second /*timeout*/)
	if err != nil {
		panic(err)
	}

	buf := bufio.NewReader(c)
	_, err = c.Write([]byte("AUTH " + config.passwd + "\r\n"))
	if err != nil {
		panic(err)
	}
	buf.ReadString('\n')

	c.Write([]byte("proxyId" + "\r\n"))
	buf.ReadString('\n')
	id, err := buf.ReadString('\n')
	if err != nil {
		panic(err)
	}
	proxyid := id[:len(id)-2] // remove '\r\n'
	return &InterfaceConn{conn: c, proxyId: proxyid}
}

func findAllInterface(config *RedisConfig) map[string]*InterfaceConn {
	confMap := map[string]*InterfaceConn{}
	for i := 0; i < 300; i++ {
		ifConn := getNewConn(config)
		if _, ok := confMap[ifConn.proxyId]; ok {
			ifConn.conn.Close()
		} else {
			confMap[ifConn.proxyId] = ifConn
		}
	}
	return confMap
}

// monitor starts outout commands in monitor
func monitor(conn net.Conn, wg *sync.WaitGroup) {
	buf := bufio.NewReader(conn)
	conn.Write([]byte("monitor" + "\r\n"))

	for {
		str, err := buf.ReadString('\n')

		if len(str) > 0 {
			if str == "+OK\r\n" {
				continue
			}

			if strings.HasPrefix(str, "-ERR ") {
				fmt.Printf("Redis return error message: " + strings.TrimRight(str[1:], "\r\n"))
				wg.Done()
			}

			fmt.Printf("%s", str[1:])
		}

		if err != nil {
			fmt.Println(err.Error())
			wg.Done()
		}
	}
}

func main() {
	var wg sync.WaitGroup
	err, redisConfig := getRedisConfig()
	if err != nil {
		return
	}
	confMap := findAllInterface(redisConfig)
	for _, ifConn := range confMap {
		conn := ifConn.conn
		wg.Add(1)
		go monitor(conn, &wg)
	}
	wg.Wait()
	return
}
